#!/usr/bin/perl

package Sympa::Template::Plugin::Sponsoring::Commons;
our $VERSION = 0.01;

use strict;
use warnings;
use Sympa::Constants;
use Sympa::Log;
use Sympa::List;

sub new {
	my ($class, @params) = @_;
	my $self = {
		log => Sympa::Log->instance,,
		sdm => Sympa::DatabaseManager->instance,
	};
	$self->{database}{godsons} = 'plugin_godsons';
	$self->{database}{old_sponsors} = 'plugin_godsons_old_sponsors';
	$self->{table_structure} = 
	{
		$self->{database}{godsons} => {
			$self->{database}{godsons}.'_sponsor' => {
				key => 1,
				type => 'varchar(150)',
				notnull => 1
			},
			$self->{database}{godsons}.'_list' => {
				type => 'varchar(100)',
				key => 1,
				notnull => 1,
			},
			$self->{database}{godsons}.'_godson' => {
				key => 1,
				type => 'varchar(150)',
				notnull => 1,
			},
		},
		$self->{database}{old_sponsors} => {
			$self->{database}{old_sponsors}.'_sponsor' => {
				type => 'varchar(100)',
				key => 1,
				notnull => 1,
			},
			$self->{database}{old_sponsors}.'_sponsored_list' => {
				type => 'varchar(100)',
				key => 1,
				notnull => 1,
			},
			$self->{database}{old_sponsors}.'_date_removal' => {
				type => 'datetime',
				notnull => 1,
			},
		},
	};

	my $config_structure = {
		'godfathered_list' => {
			'occurrence' => '0-n',
			'format' => {
				listname => {
					'format' => '.+',
					'occurrence' => '1',
					'case' => 'insensitive',
				},
				host => {
					'format' => '.+',
					'occurrence' => '1',
					'case' => 'insensitive',
				},
				expiration_period => {
					'format' => '\d+',
					'occurrence' => '1',
					'case' => 'insensitive',
				},
				sponsor_lists => {
					'format' => '.+',
					'occurrence' => '1-n',
					'case' => 'insensitive',
				},
				max_godson_nb => {
					'format' => '\d+',
					'occurrence' => '1',
					'case' => 'insensitive',
				},
				exclusive_sponsoring => {
					'format' => '\d+',
					'occurrence' => '0-1',
				},
			}
		}
	};

	my $file = $class;
	$file =~ s/::/\//g;
	my $config_file = Sympa::Constants::MODULEDIR.'/'.$file.'.conf';
	
	my $config = Conf::load_generic_conf_file($config_file, $config_structure, 'abort');
	return $class->error('Config error') unless($config);
	
	foreach my $key (keys %$config) {
		if($key eq 'godfathered_list') {
			$self->{config}{godfathered_lists} = {};
			foreach my $godfathered_list (@{$config->{$key}}) {
				$self->{config}{godfathered_lists}{$godfathered_list->{listname}} = $godfathered_list;
			}
		}else{
			$self->{config}{$key} = $config->{$key};
		}
	}
	
	foreach my $key (keys %{$self->{config}{godfathered_lists}}) {
		if($self->{config}{godfathered_lists}{$key}{sponsor_lists}){
			$self->{config}{godfathered_lists}{$key}{sponsor_lists} =~ s{\s}{}g;
			my @sponsors = split ',',$self->{config}{godfathered_lists}{$key}{sponsor_lists};
			$self->{config}{godfathered_lists}{$key}{sponsor_lists} = \@sponsors;
		}
	}
	
	bless $self,$class;
	
	my $init = $self->_init();
	return $init unless ($init == 1);

	return $self;
}

# subroutine is_allowed_sponsor #
# check wether the logged user is a sponsor for the list
# return 1 if sponsor is allowed, 0 otherwise.

sub is_allowed_sponsor {
	my ($self,$user_email,$listname) = @_;

	return 0 unless ($listname);
    foreach my $sponsor_list (@{$self->{config}{godfathered_lists}{$listname}{sponsor_lists}}) {
		my $sponsor_list_object;
		if ($sponsor_list_object = Sympa::List->new($sponsor_list)) {
			if($sponsor_list_object->is_list_member($user_email)) {
				return 1;
			}
		} else {
			$self->{log}->syslog('err', "[Sponsoring] $0 : Sympa server doesn\'t know $sponsor_list list, which is recorded as a list of sponsors for $listname list.");	    
		}
    }
    return 0;
}

sub set_maverick {
	my ($self,$user_email,$listname,$date) = @_;

	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			INSERT INTO %s  (%s,%s,%s) VALUES(?,?,?)
			},
			$self->{database}{old_sponsors},
			$self->{database}{old_sponsors}.'_sponsor',
			$self->{database}{old_sponsors}.'_sponsored_list',
			$self->{database}{old_sponsors}.'_date_removal',
		),
		$user_email,
		$listname,
		$date,
	);
}

sub unset_maverick {
	my ($self,$user_email,$listname) = @_;

	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			DELETE FROM %s WHERE %s=? AND %s=?
			},
			$self->{database}{old_sponsors},
			$self->{database}{old_sponsors}.'_sponsor',
			$self->{database}{old_sponsors}.'_sponsored_list',
		),
		$user_email,
		$listname,
	);
}

sub is_maverick {
	my ($self,$sponsor_email,$sponsored_list) = @_;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			SELECT %s from %s WHERE (%s=? AND %s=?)
			},
			$self->{database}{old_sponsors}.'_sponsor',
			$self->{database}{old_sponsors},
			$self->{database}{old_sponsors}.'_sponsor',
			$self->{database}{old_sponsors}.'_sponsored_list',),
		$sponsor_email,
		$sponsored_list,
		);
	if (my $godson = $sth->fetchrow_arrayref()) {
		return 1;
	}
	return 0;
}

sub get_godsons {
	my ($self,$sponsor_email,$sponsored_list) = @_;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			SELECT %s from %s WHERE (%s=? AND %s=?)
			},
			$self->{database}{godsons}.'_godson',
			$self->{database}{godsons},
			$self->{database}{godsons}.'_sponsor',
			$self->{database}{godsons}.'_list',),
		$sponsor_email,
		$sponsored_list,
		);
	my $result;
	$result->{checked_godsons} = 1;
	$result->{sponsor_email} = $sponsor_email;
	$result->{nb_godsons} = 0;
	while (my $godson = $sth->fetchrow_arrayref()) {
		push @{$result->{godsons_email}}, $godson->[0];
		$result->{nb_godsons}++;
	}
	return $result;
}

sub get_sponsor {
	my ($self,$godson_email,$sponsored_list) = @_;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			SELECT %s from %s WHERE (%s=? AND %s=?)
			},
			$self->{database}{godsons}.'_sponsor',
			$self->{database}{godsons},
			$self->{database}{godsons}.'_godson',
			$self->{database}{godsons}.'_list',),
		$godson_email,
		$sponsored_list,
		);
	if (my $sponsor = $sth->fetchrow_arrayref()) {
		return {
			checked_sponsor => 1,
			godson_email => $godson_email,
			sponsor_email => $sponsor->[0]
		};
	}
	return {
		checked_sponsor => 1,
		no_sponsor => 1,
		godson_email => $godson_email,
	};
	
}

sub get_all_sponsors {
	my ($self) = @_;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			SELECT DISTINCT %s, %s from %s
			},
			$self->{database}{godsons}.'_sponsor',
			$self->{database}{godsons}.'_list',
			$self->{database}{godsons},
			),
		);
	my %sponsors;
	while (my $sponsor = $sth->fetchrow_hashref()) {
		$sponsors{$sponsor->{$self->{database}{godsons}.'_list'}}{$sponsor->{$self->{database}{godsons}.'_sponsor'}} = 1;
	}
	return %sponsors;
}

sub get_all_godsons {
	my ($self) = @_;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			SELECT DISTINCT %s, %s from %s
			},
			$self->{database}{godsons}.'_godson',
			$self->{database}{godsons}.'_list',
			$self->{database}{godsons},
			),
		);
	my %godsons;
	while (my $godson = $sth->fetchrow_hashref()) {
		$godsons{$godson->{$self->{database}{godsons}.'_list'}}{$godson->{$self->{database}{godsons}.'_godson'}} = 1;
	}
	return %godsons;
}

sub add_godson {
	my ($self,$godson_email,$send_mail_to_added,$sponsored_list,$sponsor) = @_;
	unless ($godson_email) {
		die 'L\'email du filleul à ajouter est manquant';
	}
	if ($self->is_sponsored($godson_email,$sponsored_list)) {
		die "L'utilisateur $godson_email est déjà parrainé.";
	}
	$godson_email =~ s{\s}{}g;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			INSERT INTO %s (%s,%s,%s) VALUES(?,?,?)
			},
			$self->{database}{godsons},
			$self->{database}{godsons}.'_sponsor',
			$self->{database}{godsons}.'_godson',
			$self->{database}{godsons}.'_list',),
		$sponsor,
		$godson_email,
		$sponsored_list,
		);
	if ($send_mail_to_added) {
		my $list = Sympa::List->new($sponsored_list);
        Sympa::send_notify_to_user($list, 'plugin', $godson_email,
            {entry => 'sponsoring_welcome_message.tt2',sponsor => $sponsor})
            or die 'Unable to send notification to new subscriber';
	}
	return 1;
}

sub del_godson {
	my ($self,$godson_email,$send_mail_to_deleted,$sponsored_list,$sponsor) = @_;
	$self->{log}->syslog('info', "Deleting %s (warn: %s)", $godson_email,$send_mail_to_deleted);
	unless ($godson_email) {
		die 'L\'email du filleul à supprimer est manquant';
	}
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			DELETE from %s WHERE (%s=? AND %s=? AND %s=?)
			},
			$self->{database}{godsons},
			$self->{database}{godsons}.'_sponsor',
			$self->{database}{godsons}.'_godson',
			$self->{database}{godsons}.'_list',),
		$sponsor,
		$godson_email,
		$sponsored_list,
		);

	if ($send_mail_to_deleted) {
		my $list = Sympa::List->new($sponsored_list);
        Sympa::send_notify_to_user($list, 'plugin', $godson_email,
            {entry => 'sponsoring_bye_message.tt2',sponsor => $sponsor,expiration_period => $self->{config}{godfathered_lists}{$sponsored_list}{expiration_period}})
            or die sprintf 'Unable to send email to %s',$godson_email;
	}
	$self->{log}->syslog('info', "Deleted %s (warn: %s)", $godson_email,$send_mail_to_deleted);
	return 1;
}

sub is_list_sponsored {
	my ($self,$listname) = @_;
	unless ($self->{config}{godfathered_lists}{$listname}) {
		return 0;
	}else{
		return 1;
	}
}

sub get_sponsored_lists {
	my ($self) = @_;
	return $self->{config}{godfathered_lists};
}

sub get_removed_sponsors {
	my ($self) = @_;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			SELECT * from %s
			},
			$self->{database}{old_sponsors},
			),
		);
	my @result;
	while (my $old_sponsor = $sth->fetchrow_hashref()) {
		push @result, {
			sponsor_email => $old_sponsor->{$self->{database}{old_sponsors}.'_sponsor'},
			list => $old_sponsor->{$self->{database}{old_sponsors}.'_sponsored_list'},
			removed_since => $old_sponsor->{$self->{database}{old_sponsors}.'_date_removal'},
		};
	}
	return @result;
}

sub get_expired_sponsors {
	my ($self) = @_;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			SELECT * from %s
			},
			$self->{database}{old_sponsors},
			),
		);
	my @result;
	while (my $old_sponsor = $sth->fetchrow_hashref()) {
		if (is_older_than($old_sponsor->{$self->{database}{old_sponsors}.'_date_removal'},$self->{config}{godfathered_lists}{$old_sponsor->{$self->{database}{old_sponsors}.'_sponsored_list'}}{expiration_period})) {
			push @result, {
				sponsor_email => $old_sponsor->{$self->{database}{old_sponsors}.'_sponsor'},
				list => $old_sponsor->{$self->{database}{old_sponsors}.'_sponsored_list'},
				removed_since => $old_sponsor->{$self->{database}{old_sponsors}.'_date_removal'},
			};
		}
	}
	return @result;
}

sub del_expired_sponsor {
	my ($self,$sponsor_email, $sponsored_list) = @_;
	$self->{sdm}->do_prepared_query(
		sprintf (q{
			DELETE FROM %s WHERE %s = ? AND %s = ?
			},
			$self->{database}{old_sponsors},
			$self->{database}{old_sponsors}.'_sponsor',
			$self->{database}{old_sponsors}.'_sponsored_list',
			),
			$sponsor_email,
			$sponsored_list,
		);
	
}

sub is_older_than {
    my ($date,$expiration_period) = @_;
    my $return_value;
    my $One_month_in_epoch = $expiration_period*24*60*60;

    my $now_epoch_time = time();
    $date =~ /(\d\d\d\d)-(\d\d)-(\d\d)/;
    my $epoch_date =  Time::Local::timelocal(0,0,0,$3,$2 - 1,$1 - 1900);
    
    if ( ($epoch_date + $One_month_in_epoch) < $now_epoch_time ) {
	$return_value = 1;
    }

    return $return_value;
}

sub is_sponsored {
	my ($self, $email, $sponsored_list) = @_;
	my $sth = $self->{sdm}->do_prepared_query(
		sprintf (q{
			SELECT * from %s WHERE (%s=? AND %s=?)
			},
			$self->{database}{godsons},
			$self->{database}{godsons}.'_godson',
			$self->{database}{godsons}.'_list',),
		$email,
		$sponsored_list,
		);
	my @result;
	while (my $godson = $sth->fetchrow_hashref()) {
		push @result, $godson;
	}
	return @result;
}

## PRIVATE METHODS

sub _init {
	my ($self) = @_;
	
	unless ($self->_check_version()) {
		my $table_exists;
		my $class = ref $self;
		my @tables;
		my %table_exists;
		unless (
				$self->{sdm}
				and @tables = @{$self->{sdm}->get_tables()}
				) {
			die 'Unable to query database.';
		}
		foreach my $table (@tables) {
			$table_exists{$table} = 1;
		}
		foreach my $plugin_table (keys %{$self->{table_structure}}) {
			unless ($table_exists{$plugin_table}) {
				unless (
						$self->{sdm}
						and $self->{sdm}->add_table({table => $plugin_table})
						) {
					die "Unable to create table $plugin_table.";
				}
			}
			my @primary_key;
			my $existing_fields = $self->{sdm}->get_fields({table => $plugin_table});
			foreach my $field (keys %{$self->{table_structure}{$plugin_table}}) {
				my $field_description = {%{$self->{table_structure}{$plugin_table}{$field}},field => $field, table => $plugin_table};
				if ($existing_fields->{$field}) {
					unless ($self->{sdm}->update_field($field_description)) {
						die sprintf 'Unable to update field %s: %s',$field,$self->{sdm}->error;
					}
				}else{
					unless ($self->{sdm}->add_field($field_description)) {
						die sprintf 'Unable to add field %s: %s',$field,$self->{sdm}->error;
					}
				}
				if ($field_description->{key}) {
					push @primary_key, $field;
				}
			}
			
			unless ($table_exists{$plugin_table}) {
				$self->{sdm}->delete_field({
					table => $plugin_table,
					field => 'temporary',
				});
			}
			my $primary_keys = $self->{sdm}->get_primary_key({table => $plugin_table});
			if(keys %$primary_keys) {
				unless ($self->{sdm}->unset_primary_key({table => $plugin_table})) {
					die "Unable to unset primary key";
				}
			}
			unless (my $report = $self->{sdm}->set_primary_key ({
				table => $plugin_table,
				fields => \@primary_key,
			})) {
				die $report;
			}
		}
	}
	return 1;
}

sub _check_version {
	my ($self) = @_;
	my $current_version;
	eval {$current_version = $self->_read_version()};
	if ($@ or $current_version ne $VERSION) {
		print STDERR "No version or current version too old.\n";
		$self->_write_version($VERSION);
		return 0;
	}
	return 1;
}

sub _read_version {
	my ($self) = @_;
	my $file_root = ref $self;
	$file_root =~ s{::}{/}g;
	my $version_file = Sympa::Constants::MODULEDIR.'/'.$file_root.'.version';
	open my $fh, "$version_file"
		or die "Unable to get read access to file $version_file";
	my $version = <$fh>;
	close $fh;
	$version =~ s{\s}{}g;
	return $version;
}

sub _write_version {
	my ($self,$version_number) = @_;
	my $file_root = ref $self;
	$file_root =~ s{::}{/}g;
	my $version_file = Sympa::Constants::MODULEDIR.'/'.$file_root.'.version';
	open my $fh, ">$version_file"
		or die "Unable to write version to file $version_file";
	print $fh $version_number;
	close $fh;
	Sympa::Tools::File::set_file_rights(
		(
			user => Sympa::Constants::USER(),
			group => Sympa::Constants::GROUP(),
			file => $version_file,
		)
	);
}


1;
