#!/usr/bin/perl

use lib split(/:/, $ENV{SYMPALIB} || ''), '/usr/local/sympa-stable/bin';
use strict;
use warnings;
use English;

use Sympa::Alarm;
use Conf;
use Sympa::Constants;
use Sympa::Log;
use Sympa::Process;
use Sympa::Template::Plugin::Sponsoring;

my $process = Sympa::Process->instance;
$process->init(pidname => 'check_godsons');

# Set the User ID & Group ID for the process
$GID = $EGID = (getgrnam(Sympa::Constants::GROUP))[2];
$UID = $EUID = (getpwnam(Sympa::Constants::USER))[2];

## Required on FreeBSD to change ALL IDs
## (effective UID + real UID + saved UID)
POSIX::setuid((getpwnam(Sympa::Constants::USER))[2]);
POSIX::setgid((getgrnam(Sympa::Constants::GROUP))[2]);

## Check if the UID has correctly been set (useful on OS X)
unless (($GID == (getgrnam(Sympa::Constants::GROUP))[2])
    && ($UID == (getpwnam(Sympa::Constants::USER))[2])) {
    die
        "Failed to change process user ID and group ID. Note that on some OS Perl scripts can't change their real UID. In such circumstances Sympa should be run via sudo.\n";
}

# Load sympa.conf
unless (Conf::load()) {
    die sprintf
        "Unable to load Sympa configuration, file %s or one of the virtual host robot.conf files contain errors. Exiting.\n",
        Conf::get_sympa_conf();
}

my $sponsoring = Sympa::Template::Plugin::Sponsoring::Commons->new();

# Restore sponsors who are not maverick anymore.
foreach my $old_sponsor ($sponsoring->get_removed_sponsors()) {
	if ($sponsoring->is_allowed_sponsor($old_sponsor->{sponsor_email},$old_sponsor->{list})) {
		my $list_object = Sympa::List->new($old_sponsor->{list});
		$sponsoring->unset_maverick($old_sponsor->{sponsor_email},$old_sponsor->{list});
		my $endangered_godsons = $sponsoring->get_godsons($old_sponsor->{sponsor_email},$old_sponsor->{list});
		foreach my $endangered_godson (@{$endangered_godsons->{godsons_email}}) {
			Sympa::send_notify_to_user($list_object, 'plugin', $endangered_godson,
				{
					entry => 'sponsoring_sponsor_is_back_message.tt2',
					sponsor => $old_sponsor->{sponsor_email},
					expiration_period => $sponsoring->{config}{godfathered_lists}{$old_sponsor->{list}}{expiration_period}
				})
			or die sprintf 'Unable to send email to %s',$endangered_godson;
		}
	}
}

## Remove godsons from too old expired sponsors. Clean expired sponsors table aterwards.
my @old_sponsors = $sponsoring->get_expired_sponsors();
foreach my $expired_sponsor(@old_sponsors) {
	my $godsons_to_remove = $sponsoring->get_godsons($expired_sponsor->{sponsor_email}, $expired_sponsor->{list});
	foreach my $godson_to_remove (@{$godsons_to_remove->{godsons_email}}) {
		$sponsoring->del_godson($godson_to_remove, 1, $expired_sponsor->{list},$expired_sponsor->{sponsor_email});
	}
	$sponsoring->del_expired_sponsor($expired_sponsor->{sponsor_email}, $expired_sponsor->{list});
}

# 2. note all sponsors who have disappeared from sponsored lists
# 2.1 retrieve all (sponsor, sponsored list) couples from godsons table "SELECT distinct parrain, liste FROM $godsons_table"
my %sponsors = $sponsoring->get_all_sponsors();
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);  
$year += 1900;  
$mon += 1;
my $removing_date = "$year-$mon-$mday";
foreach my $list (keys %sponsors) {
	my @maverick_sponsors;
	# 2.2 get all sponsors who are no more subscriber of a sponsored list
	foreach my $sponsor ( keys %{$sponsors{$list}} ){
		unless ($sponsoring->is_allowed_sponsor($sponsor,$list)) {
			push @maverick_sponsors, $sponsor;
		}
	}
	my $list_object = Sympa::List->new($list);
	foreach my $maverick (@maverick_sponsors) {
		unless ($sponsoring->is_maverick($maverick, $list)) {
			$sponsoring->set_maverick($maverick, $list, $removing_date);
			my $endangered_godsons = $sponsoring->get_godsons($maverick,$list);
			foreach my $endangered_godson (@{$endangered_godsons->{godsons_email}}) {
				Sympa::send_notify_to_user($list_object, 'plugin', $endangered_godson,
					{
						entry => 'sponsoring_no_sponsor_message.tt2',
						sponsor => $maverick,
						expiration_period => $sponsoring->{config}{godfathered_lists}{$list}{expiration_period}
					})
				or die sprintf 'Unable to send email to %s',$endangered_godson;
			}
		}
	}
}

my %godsons = $sponsoring->get_all_godsons();
my $sponsored_lists = $sponsoring->get_sponsored_lists;
foreach my $list (keys %godsons) {
	my $list_object = Sympa::List->new($list);
	next unless ($sponsored_lists->{$list}{exclusive_sponsoring});
	my @sponsor_lists;
	foreach my $sponsor_list (@{$sponsored_lists->{$list}{sponsor_lists}}) {
		push @sponsor_lists, Sympa::List->new($sponsor_list);
	}
	foreach my $godson (keys %{$godsons{$list}}) {
		foreach my $sponsor_list (@sponsor_lists) {
			if($sponsor_list->is_list_member($godson)) {
				$sponsoring->{log}->syslog('info', 'list "%s", godson "%s" is subscriber to "%s". Removing from godsons.',$list, $godson, $sponsor_list->get_id());
				my $sponsor = $sponsoring->get_sponsor($godson,$list);
				$sponsoring->del_godson($godson,0,$list,$sponsor->{sponsor_email});
				Sympa::send_notify_to_user($list_object, 'plugin', $sponsor->{sponsor_email},
					{
						entry => 'sponsoring_no_more_godson_message.tt2',
						godson => $godson,
						sponsor_list => $sponsor_list->get_id(),
					})
				or die sprintf 'Unable to send email to %s',$sponsor->{sponsor_email};
			}
		}
	}
}
